//******************************************************************************
// WIN_01.cpp - ffnet eine eigene Client-Area
//******************************************************************************
// Include-Datei <windows.h> ist unverzichtbar, sie enthlt alle wichtigen
// defines, die fr ein Windows-Programm bentigt werden
//------------------------------------------------------------------------------
#include <windows.h>

//------------------------------------------------------------------------------
// Prototyp der Handlerfunction, die die Messages der Client-Area verarbeitet
//------------------------------------------------------------------------------
// VERALTETE DEKLARATIONSFORM:
// long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);
// Das Schlsselwort PASCAL war frher notwendig, weil der Windowskernel
// ursprnglich in Pascal programmiert wurde. Pascal und C legen die Parameter
// in unterschiedlicher Reihenfolge auf dem Stack ab, so dass der C-Funktion
// mitgeteilt werden musste, da die Parameter in anderer Reihenfolge vom Stack
// abzuholen sind. In den neueren Windowsversionen wird das gleiche durch den
// Zusatz CALLBACK erreicht
//------------------------------------------------------------------------------
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);

//------------------------------------------------------------------------------
// Windows bentigt einen eindeutigen Namen (die Fensterklasse), um feststellen
// zu knnen, ob das gleiche Programm schon luft. Ein Kopieren und Umbenennen
// der Exe-Datei gengt also nicht, um Windows zu berlisten...
// Der Sinn dahinter ist, dass bei einem erneuten Aufruf des Programms nur der
// Datenbereich und nicht der Programmbereich neu angelegt wird.                                                                   //
//------------------------------------------------------------------------------
const char sWindowclassname [] = "Win_01";

//------------------------------------------------------------------------------
// Datentyp HWND bedeutet Window-Handle, also eindeutige Kennnummer des Fensters
//------------------------------------------------------------------------------
HWND hWindow;

//------------------------------------------------------------------------------
// Alle Windowsprogramme beginnen mit "WinMain" statt mit "main"
//------------------------------------------------------------------------------
// Das pragma "argsused" ist Borland-spezifisch und unterdrckt das Warning,
// dass wir den Standardparameter "lpszCmdParam" nirgendwo benutzt haben...
//------------------------------------------------------------------------------
// hInstance:
// In diesem Parameter bergibt der Windowskernel die Programmkennung.
// Dies ist ntig, da unter Windows ja mehrere Programme parallel laufen knnen.
// Jedes Programm erhlt deshalb eine eindeutige ID-Nummer, ber welche die
// Messages zugestellt werden knnen.
//------------------------------------------------------------------------------
// hPrevInstance:
// 16-Bit Windows: In diesem Parameter teilt Windows mit, ob das gleiche
// Programm bereits im Speicher vorliegt, also erneut gestartet worden ist. In
// diesem Fall drfen einige Teile des Programmcodes nicht erneut durchlaufen
// werden. "hPrevInstance" ist NULL, wenn das Programm zum ersten Mal in den
// Speicher geladen wird, ansonsten erhlt man hier die Instanzen-Nummer
// (hInstance) des vorherigen Aufrufes.
// 32-Bit Windows: Parameter ist immer NULL
//------------------------------------------------------------------------------
// lpszCmdParam:
// enthlt die Kommandozeilen-Parameter, die ggf. beim Aufruf bergeben wurden
//------------------------------------------------------------------------------
// nCmdShow:
// legt fest, ob das Programm als Piktogramm (Icon) auf der Oberflche erscheint
// oder als Fenster.
//------------------------------------------------------------------------------

//==============================================================================
// Hauptprogramm
// VERALTETE DEKLARATIONSFORM:
// int PASCAL WinMain (HANDLEhInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine,
//                     int nCmdShow)
//==============================================================================
#pragma argsused
WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdParam,
                    int nCmdShow)
{
   //---------------------------------------------------------------------------
   // Datentyp MSG bedeutet Message, eine Struct, die die Daten einer an das
   // Programm weitergeleiteten Nachricht enthlt
   //---------------------------------------------------------------------------
   MSG aMessage;

   //---------------------------------------------------------------------------
   // Datentyp WNDCLASS meldet die Client-Area des Programms beim Windows-
   // Message-Dispatcher als Nachrichtenempfnger an
   //---------------------------------------------------------------------------
   WNDCLASS aWindowclass;

   //---------------------------------------------------------------------------
   // Die Abfrage auf hPrevInstance stellt sicher, dass der Ablaufcode eines
   // Programms immer nur einmal geladen ist. Bei Multi-Instanz-Programmen
   // wird immer nur der Datenteil neu angelegt (16Bit)
   // Unter 32-Bit-Windows ist dieser Parameter immer NULL! - sollte aber bei
   // Multi-Plattformprogrammen (Programme fr Win3.x und Win95) sicherheits-
   // halber abgefragt werden
   //---------------------------------------------------------------------------
   if (!hPrevInstance)
   {
      //------------------------------------------------------------------------
      // Wenn das Programm als erste Instanz in den Speicher geladen wird, muss
      // das Hauptfenster (die Client-Area) beim Windows-Kernel angemeldet
      // werden. Dazu ist eine WNDCLASS-Struct auszufllen, die das Fenster als
      // Fensterklasse anmeldet und ein bestimmtes Aussehen vereinbart
      //------------------------------------------------------------------------
      // Der Class-Style (CS_xxxx) legt bestimmte Verhaltensweisen des Haupt-
      // fensters (d.h. der Client-Area) fest. Am hufigsten werden verwendet:
      // CS_HREDRAW = Gibt an, dass das gesamte Fenster neu zu zeichnen ist,
      //              wenn Das Fenster in horizontaler Richtung bewegt oder
      //              in seiner Breite verndert wurde
      // CS_VREDRAW = Gibt an, dass das gesamte Fenster neu zu zeichnen ist,
      //              wenn Das Fenster in vertikaler Richtung bewegt oder
      //              in seiner Hhe verndert wurde
      // CS_NOCLOSE = Schaltet das "Schliesymbol" in der obersten Zeile ab
      // CS_DBLCLKS = Weist Windows an, eine Doppelclick-Message an die
      //              Message-Handler-Function des Programms zu schicken, wenn
      //              der Benutzer einen Doppelclick in irgendeinem der zur
      //              Anwendung gehrenden Fenster eingibt.
      // Auerdem gibt es noch weitere, seltener verwendeten Styles.
      //------------------------------------------------------------------------
      aWindowclass.style         = CS_HREDRAW | CS_VREDRAW;

      //------------------------------------------------------------------------
      // Name der Function, die in diesem Programm fr die Abarbeitung der
      // Messages sorgt. Die Funktion "DispatchMessage" ruft diesen Message-
      // Handler automatisch auf, weil er Teil der Klassenbeschreibung ist
      // entspricht einem Pointer auf diese Function
      //------------------------------------------------------------------------
      aWindowclass.lpfnWndProc   = WndProc;

      //------------------------------------------------------------------------
      // Speicherplatz, der fr eigene Daten im Rahmen der Fensterklasse und des
      // Fensters vom Benutzer als Erweiterung der Struktur vorgesehene sind.
      // Wird hier nicht und auch sonst selten benutzt
      //------------------------------------------------------------------------
      aWindowclass.cbClsExtra    = 0;
      aWindowclass.cbWndExtra    = 0;

      //------------------------------------------------------------------------
      // Die Erst-Instance wird ebenfalls in der Struktur gesichert, bei
      // weiteren Instancen sorgt Windows fr die Fortfhrung der Zhlerei in
      // einer Liste (schlielich muss ja auch die hPrevInstance irgendwo her-
      // kommen, d.h. Windows muss sie verwalten
      //------------------------------------------------------------------------
      aWindowclass.hInstance     = hInstance;

      //------------------------------------------------------------------------
      // Angabe des Icons, welches angezeigt wird, wenn das Programm als Icon
      // auf den Desktop heruntergeclickt wird. Hier das Standardicon
      //------------------------------------------------------------------------
      aWindowclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);

      //------------------------------------------------------------------------
      // Angabe des Mouse-Cursors, der angezeigt wird, wenn ddie Mouse sich
      // im Progframmfenster befindet. Hier der Standardcursor
      //------------------------------------------------------------------------
      aWindowclass.hCursor       = LoadCursor (NULL, IDC_ARROW);

      //------------------------------------------------------------------------
      // Angabe des Hintergrund-Pinsels (Farbe/Muster), mit dem die Flche des
      // Fensters bei Programmstart gefllt wird. Hier ein Standardbrush
      //------------------------------------------------------------------------
      aWindowclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);

      //------------------------------------------------------------------------
      // Angabe der Menstruktur, die benutzt werden soll um das Men des
      // Fensters zu bilden. Hier Null, weil kein Men verwendet wird.
      //------------------------------------------------------------------------
      aWindowclass.lpszMenuName  = NULL;

      //------------------------------------------------------------------------
      // Eintragen des Klassennamens (s.o.) zur eindeutigen Identifikation
      // Meistens wird der Name des Programms verwendet
      //------------------------------------------------------------------------
      aWindowclass.lpszClassName = sWindowclassname;

      //------------------------------------------------------------------------
      // Nachdem die Struktur zur Klasseneintragung im Windowskernel korrekt
      // ausgefllt ist, muss es beim Windowskernel auch registriert werden
      //------------------------------------------------------------------------
      if (!RegisterClass (&aWindowclass))
      {
         MessageBox (NULL,"Registrierung nicht mglich",sWindowclassname,MB_OK);
         return 0;
      }
   }

   //---------------------------------------------------------------------------
   // Nachdem wir die Klasse registriert haben, ist der  Fenstertyp beim Kernel
   // bekannt. Um nun ein Fenster dieses Typs zu ffnen rufen wir die Function
   // CreateWindow auf. Erst  dadurch wird intern der Speicherplatz geschaffen,
   // um ein Fenster zu verwalten.
   // Das erste Fenster (Hauptfenster) eines Programms ist zugleich das
   // wichtigste, denn es ist Bezugspunkt fr alle weiteren Unterfenster.
   // Das Hauptfenster wird daher auch Kundenbereich (Client-Area) genannt.
   // Zudem knnen wir in groben Zgen festlegen, wie das Fenster aussehen soll.
   // Als Ergebnis der Funktion erhalten wir ein Handle auf das erzeugte
   // Fenster, also eine eindeutige Identifikationsnummer.
   //---------------------------------------------------------------------------
   // Parameter 1: Name der Fensterklasse, zu dem die Client-Area gehrt
   //---------------------------------------------------------------------------
   // Parameter 2: Text, der in der Titelzeile des Fensters steht.
   //---------------------------------------------------------------------------
   // Parameter 3: Fensterstil. Mgliche Werte sind
   //              (mit dem binr-Oder-Operator verbunden):
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // Titelzeilen-Elemente
   // WS_CAPTION          Fenster mit Titelzeile (beinhaltet WS_BORDER Stil)
   // WS_MAXIMIZEBOX      Fenster hat Vergrerungsbutton in Titelzeile
   // WS_MINIMIZEBOX      Fenster hat Verkleinerungsbutton in der Titelzeile
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // Randelemente
   // WS_BORDER	          Fenster mit schmalem Rand
   // WS_THICKFRAME       Fenster mit breitem Rand, ber den Fenstergre
   //                     eingestellt werden kann. Identisch mit WS_SIZEBOX
   // WS_SIZEBOX          Siehe WS_THICKFRAME
   // WS_HSCROLL          Erzeugt ein Fenster mit einer senkrechten Scrollbar
   // WS_VSCROLL          Erzeugt ein Fenster mit einer horizontal-Scrollbar
   // WS_SYSMENU          Erzeugt ein Fenster mit dem System-Men in der
   //                     Titelzeile. Dazu muss Stil WS_CAPTION gesetzt sein
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // Zusammengefate Elemente
   // WS_OVERLAPPED       Erzeugt sogenanntes OVERLAPPED-Fenster. Hat immer
   //                     Titelzeile (WS_CAPTION) und Rand (WS_BORDER).
   //                     Identisch mit WS_TILED
   // WS_TILED            Siehe WS_OVERLAPPED
   // WS_DLGFRAME         Fenster mit dem typischen Aussehen einer Dialogbox
   //                     Kann keine Titelzeile (WS_CAPTION) haben!
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // Darstellungsgren
   // WS_ICONIC           Erzeugt Fenster, das zu Beginn als Icon vorliegt.
   // WS_MAXIMIZE         Erzeugt ein Fenster maximaler Gre.
   // WS_MINIMIZE         Identisch mit WS_ICONIC
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // sonstige Eigenschaften
   // WS_DISABLED         Fenster ist abgeschaltet
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // Zusammenfassende Stil-Pakete
   // WS_OVERLAPPEDWINDOW Erzeugt ein Fenster mit den Stilen WS_OVERLAPPED,
   //                     WS_CAPTION, WS_SYSMENU, WS_THICKFRAME,
   //                     WS_MINIMIZEBOX, und WS_MAXIMIZEBOX.
   // WS_TILEDWINDOW      Identisch mit WS_OVERLAPPEDWINDOW
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // Stile fr abhngige Fenster (Popupfenster / Childfenster)
   // WS_CHILD            Definiert ein Unterfenster (nicht bei Client-Area
   //                     mglich!) Schliet sich mit WS_POPUP gegenseitig aus
   // WS_CHILDWINDOW      Identisch mit WS_CHILD
   // WS_CLIPCHILDREN     Die Bereiche von Child-Fenstern werden ausgelassen,
   //                     wenn das bergeordnete Fenster neu gezeichnet wird.
   // WS_CLIPSIBLINGS     Die Bereiche von berlappenden Child-Fenstern
   //                     werden beachtet, so dass nicht in die berlappenden
   //                     Bereiche geschrieben/gezeichnet wird
   // WS_POPUP            Erzeugt ein Dialogfenster. (nicht fr Client-Area
   //                     mglich!) Schliet sich mit WS_CHILD gegenseitig aus
   // WS_POPUPWINDOW      Erzeugt ein Dialogfenster mit Standardaussehen.
   //                     Beinhaltet WS_BORDER, WS_POPUP, und WS_SYSMENU.
   //                     Muss mit WS_CAPTION kombiniert werden, um das
   //                     System-Men sichtbar zu machen
   // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   // Stile fr Dialogelemente
   // WS_GROUP            Bezeichnet das erste Fenster einer Child-Gruppe.
   //                     Wird fr die Fenster von Buttons und andere Dialog-
   //                     elemente verwendet. Zwischen Fenstern der Gruppe
   //                     kann der Focus mit den Cursortasten bewegt werden.
   //                     Das nchste Fenster mit dem WS_GROUP beendet die
   //                     Gruppe wieder und startet eine neue
   // WS_TABSTOP          Besagt, dass das Dialogelement mittels Tabulatortaste
   //                     erreicht werden kann.
   // WS_VISIBLE          Erzeugt sichtbares Dialogelement. Unsichtbare
   //                     Elemente sind zugleich auf WS_DISABLED geschaltet
   //---------------------------------------------------------------------------
   // Parameter 4: Fensterposition X-Koordinate. Angabe in Pixel. Die Angabe
   //              von CW_USEDEFAULT berlt die Entscheidung dem  Kernel
   //---------------------------------------------------------------------------
   // Parameter 5: Fensterposition Y-Koordinate. Angabe in Pixel. Die Angabe
   //              von CW_USEDEFAULT berlt die Entscheidung dem  Kernel
   //---------------------------------------------------------------------------
   // Parameter 6: Fensterbreite in Pixeln. Die Angabe von CW_USEDEFAULT
   //              berlt die Entscheidung dem Kernel
   //---------------------------------------------------------------------------
   // Parameter 7: Fensterhhe in Pixeln. Die Angabe von CW_USEDEFAULT berlt
   //              die Entscheidung dem Kernel
   //---------------------------------------------------------------------------
   // Parameter 8: Handle des bergeordneten Fensters. Da es sich hier um die
   //              Client-Area handelt, gibt es kein bergeordnetes Fenster
   //---------------------------------------------------------------------------
   // Parameter 9: Handle des Mens, welches im Fenster dargestellt werden soll.
   //              Da hier kein Men vereinbart wurde ist der Wert logischer-
   //              weise NULL. Bei CHILD-Fenstern wird dieser Eintrag als ein-
   //              deutige ID-Nummer benutzt
   //---------------------------------------------------------------------------
   // Parameter 10: Instanzenhandle. Hier wird eingetragen, zu welcher Aufruf-
   //               instanz das Fenster gehrt. Das ist wichtig, damit in  den
   //               Dialogen die richtigen Datenbereiche angezeigt werden
   //---------------------------------------------------------------------------
   // Parameter 11. Pointer fr die in der Klasse reservierten Speicherbereiche
   //               (s.o.) wird nur selten genutzt.
   //---------------------------------------------------------------------------
   hWindow = CreateWindow (sWindowclassname, "Umrechner",
                           WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);
   if (!hWindow)
   {
      MessageBox (NULL, "Fenster nicht erzeugbar", sWindowclassname, MB_OK);
      return 0;
   }

   //---------------------------------------------------------------------------
   // Das Erzeugen des Fensters beinhaltet nicht, dass das Fenster auch ange-
   // zeigt wird. Gelegentlich will man ja etwas im Hintergrund tun und das
   // zugehrige Fenster nur bei Bedarf anzeigen. Deshalb muss das Aufblenden
   // des Fensters ausdrcklich aufgerufen werden.
   // Als Parameter bergibt man das Handle des erzeugten Fensters (Identifi-
   // kationsnummer) und den Darstellungsmodus, der als Eigenschaft (z.B.
   // "als Icon") von Windows als Parameter an WinMain bergeben wurde.
   // Dieser Befehl darf nur EINMAL, hier im WinMain verwendet werden)
   //---------------------------------------------------------------------------
   ShowWindow (hWindow, nCmdShow);

   //---------------------------------------------------------------------------
   // Das Aufblenden des Fensters zeigt im Grunde nur die Flche an. Aus
   // Geschwindigkeitsgrnden werden die Fensterinhalte nur bei Bedarf oder
   // Aufforderung gemalt. Die folgende Zeile malt das ganze Fenster neu.
   // (Es wird eine WM_PAINT-Message ausgelst - s.u.)
   //---------------------------------------------------------------------------
   if (!UpdateWindow (hWindow))
   {
      MessageBox (NULL, "WM_PAINT fehlgeschlagen", sWindowclassname, MB_OK);
      return 0;
   }

   //---------------------------------------------------------------------------
   // Das Fenster ist bereit und gezeichnet, jetzt mssen wir auf alle an-
   // kommenden Botschaften reagieren. GetMessage gibt solange TRUE zurck, bis
   // eine WM_QUIT-Message anzeigt, dass das Programm beendet wurde. Die
   // WM_QUIT-Message ist die letzte Message, die ein Programm erhlt.
   // Bei ungltigen Messages (an nicht mehr vorhandenes Window-Handle) ergibt
   // GetMessage den Wert -1
   //---------------------------------------------------------------------------
   while (GetMessage (&aMessage, NULL, 0, 0))
   {
      //------------------------------------------------------------------------
      // TranslateMessage bersetzt die Windows-Tastaturcodes (Virtual Keys)
      // in Characterdaten. Natrlich nur, wenn solche Daten auch in der
      // Message vorliegen. Die MUSS gemacht werden, da man sonst mit den Key-
      // messages nichts anfangen kann.
      //------------------------------------------------------------------------
      TranslateMessage (&aMessage);

      //------------------------------------------------------------------------
      // DispatchMessage gibt die Message-struct an den Handler weiter, der in
      // der Windowsklasse eingetragen wurde (s.o.). Erst dort wird die Nach-
      // richt verarbeitet. Sind mehrere Fenster vorhanden, so entscheidet der
      // Dispatcher anhand des in der Message eingetragenen Handles, an welche
      // Handlerroutine die Message geht.
      //------------------------------------------------------------------------
      DispatchMessage (&aMessage);
   }

   //---------------------------------------------------------------------------
   // Zu guter letzt, bevor wir uns verabschieden teilen wir dem Windowskernel
   // noch mit, dass wir die WM_QUIT-Message verarbeitet haben, indem wir den
   // wparam der Message zurckgeben.
   //---------------------------------------------------------------------------
   return aMessage.wParam;
}

//==============================================================================
// Handlerfunktion der Client-Area, wird von DispatchMessage aufgerufen.
// Parameter "hwnd" gibt an, an welches Fensterhandle sich die Botschaft richtet
// Parameter "message" gibt die Messageart an
// Parameter "wParam" ist ein Datenfeld, das abhngig von der Messageart zu
//                    interpretieren ist.
// Parameter "lParam" ist ein Datenfeld, das abhngig von der Messageart zu
//                    interpretieren ist.
//==============================================================================
LRESULT CALLBACK WndProc (HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
   //---------------------------------------------------------------------------
   // Handle des Devicecontextes. Der DC kmmert sich um die Belange des anzu-
   // sprechenden Gertes (hier Bildschirm), so dass wir uns nicht selbst um die
   // Umrechnung von Farben etc. auf die Bildschirmauflsung und Farbtiefe
   // kmmern mssen. Dadurch funktionieren alle Windowsprogramme meist ohne
   // Probleme mit fast jeglicher, windowsfhiger Hardware. Der DC greift auf
   // die entsprechenden Treiber zu, ohne dass der Entwickler sich darum im
   // im Normalfall Gedanken machen muss.
   //---------------------------------------------------------------------------
   HDC hDC ;

   //---------------------------------------------------------------------------
   // Die Paintstruct wird von Windows bentigt, wenn ein Teil des Fensters
   // neu zu zeichnen ist.
   //---------------------------------------------------------------------------
   PAINTSTRUCT aPS ;

   //---------------------------------------------------------------------------
   // Der switch-Block reagiert auf die Messagearten, die von Windows als Nach-
   // richt geschickt werden. Hier nur zwei von SEHR VIELEN (ca. 240) verschie-
   // denen Messagearten, die es gibt. WM steht fr Windowsmessage, wobei diese
   // unter Umstnden in den Parametern wParam und lParam noch weitere Unter-
   // gruppen oder Informationen beinhalten
   //---------------------------------------------------------------------------
   switch (nMsg)
   {
      //------------------------------------------------------------------------
      // Die WM_PAINT-Message gibt an, was alles zu tun ist, wenn die Client-
      // Area neu zu zeichnen ist.
      // Windows schickt eine WM_PAINT-Message, wenn z.B. die Fenstergre
      // verndert wurde oder man das Fenster verschoben hat.
      //------------------------------------------------------------------------
      case WM_PAINT :
           //-------------------------------------------------------------------
           // BeginPaint leitet das Zeichnen ein. whrend dieses Vorgangs kann
           // in Windows kaum etwas anderes gemacht werden, der Bildschirm wird
           // kurzfristig exclusiv fr dieses Programm gebraucht.
           // NIEMALS hier einen Breakpoint setzen, denn das fhrt in 99% aller
           // Flle zu einem Absturz!
           //-------------------------------------------------------------------
           hDC = BeginPaint (hWnd, &aPS);

           //-------------------------------------------------------------------
           // Bendet die geschtzte Zeichnerei
           //-------------------------------------------------------------------
			     EndPaint (hWnd, &aPS);

           //-------------------------------------------------------------------
			     // Abarbeitung der WM_PAINT-Message an Windowskernel besttigen
           //-------------------------------------------------------------------
           return 0;

      //------------------------------------------------------------------------
      // Die WM_DESTROY-Message wird empfangen, also der Wunsch des Benutzers
      // das Programm zu ebeenden. ggf. ist dies nicht mglich, dann kann die
      // Beendigung hier unterbunden werden. Ist die Beendigung erlaubt,  so ist
      // hier die letzte Mglichkeit Programmresourcen und Speicherplatz freizu-
      // geben, die dynamisch erzeugt worden sind !
      //------------------------------------------------------------------------
      case WM_DESTROY :
           //-------------------------------------------------------------------
           // Wenn nichts dagegen spricht, dann hier das Programmende durch-
           // fhren. PostQuitMessage schickt eine WM-QUIT-Message an unser
           // eigenes Programm, so dass die Handler-Schleife in WinMain (die
           // GetMessage-Schleife) beendet wird
           //-------------------------------------------------------------------
           PostQuitMessage (0);

           //-------------------------------------------------------------------
           // Abarbeitung der WM_DESTROY-Message an Windeowskernel besttigen
           //-------------------------------------------------------------------
           return 0;
   }

   //---------------------------------------------------------------------------
   // da uns alle anderen Messages nicht interessieren, mssen wir dem Kernel
   // mitteilen, dass es die Nachricht als erledigt betrachten kann. Dazu werden
   // sie formal in einem Standard-Handler des Kernels weiterverarbeitet.
   //---------------------------------------------------------------------------
   return DefWindowProc (hWnd, nMsg, wParam, lParam);
}