//******************************************************************************
// WIN_08.cpp - ffnet eine eigene Client-Area und gibt dort einen Text aus
//******************************************************************************
#include <windows.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>

//==============================================================================
// Funktionsprototypen
//==============================================================================
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);

LRESULT fnWMCreate  (HWND, WPARAM, LPARAM);
LRESULT fnWMDestroy (HWND, WPARAM, LPARAM);
LRESULT fnWMPaint   (HWND, WPARAM, LPARAM);
LRESULT fnWMSize    (HWND, WPARAM, LPARAM);
LRESULT fnWMTimer   (HWND, WPARAM, LPARAM);

//------------------------------------------------------------------------------
void   fnValueTable (HDC hDC);
double fnCalcValue  (double fValue, double fFaktor=1.0, bool bMultiply=true,
                     double fOffset=0.0, bool bAddOffset=false,
                     bool bOffsetFirst=true);

//==============================================================================
// Symbolische Konstanten
//==============================================================================
#define ID_TIMER    10000

#define EINHEITTEXTLEN 100

//==============================================================================
// globale Variablen
//==============================================================================
int nAnzZeichen, nAnzZeilen;
int nSmallCharWidth, nBigCharWidth, nCharHeight;
char sEinheit1 [EINHEITTEXTLEN] = "Zoll";
char sEinheit2 [EINHEITTEXTLEN] = "cm";

const char sWindowclassname [] = "Win_08";
HWND       hWindow;

//==============================================================================
// Hauptprogramm
//==============================================================================
#pragma argsused
WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdParam,
                    int nCmdShow)
{
   MSG      aMessage;
   WNDCLASS aWindowclass;

   if (!hPrevInstance)
   {
      aWindowclass.style         = CS_HREDRAW | CS_VREDRAW;
      aWindowclass.lpfnWndProc   = WndProc;
      aWindowclass.cbClsExtra    = 0;
      aWindowclass.cbWndExtra    = 0;
      aWindowclass.hInstance     = hInstance;
      aWindowclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
      aWindowclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
      aWindowclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
      aWindowclass.lpszMenuName  = NULL;
      aWindowclass.lpszClassName = sWindowclassname;

      if (!RegisterClass (&aWindowclass))
      {
         MessageBox (NULL,"Registrierung nicht mglich",sWindowclassname,MB_OK);
         return 0;
      }
   }

   //---------------------------------------------------------------------------
   hWindow = CreateWindow (sWindowclassname, "Umrechner", WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL);
   if (!hWindow)
   {
      MessageBox (NULL, "Fenster nicht erzeugbar", sWindowclassname, MB_OK);
      return 0;
   }

   //---------------------------------------------------------------------------
   if (!SetTimer (hWindow, ID_TIMER, 1000, NULL))
   {
      MessageBox (hWindow, "Zuviele Timer!", sWindowclassname,
                  MB_ICONEXCLAMATION | MB_RETRYCANCEL);
      return 0;
   }

   //---------------------------------------------------------------------------
   ShowWindow (hWindow, nCmdShow);
   if (!UpdateWindow (hWindow))
   {
      MessageBox (NULL, "WM_PAINT fehlgeschlagen", sWindowclassname, MB_OK);
      return 0;
   }

   //---------------------------------------------------------------------------
   while (GetMessage (&aMessage, NULL, 0, 0))
   {
      TranslateMessage (&aMessage);
      DispatchMessage (&aMessage);
   }
   return aMessage.wParam;
}

//==============================================================================
// Handlerfunktion der Client-Area, wird von DispatchMessage aufgerufen.
//==============================================================================
#pragma argsused
LRESULT CALLBACK WndProc (HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
   switch (nMsg)
   {
      case WM_CREATE:  return fnWMCreate  (hWnd, wParam, lParam);
      case WM_DESTROY: return fnWMDestroy (hWnd, wParam, lParam);
      case WM_PAINT:   return fnWMPaint   (hWnd, wParam, lParam);
   	  case WM_SIZE:    return fnWMSize    (hWnd, wParam, lParam);
   	  case WM_TIMER:   return fnWMTimer   (hWnd, wParam, lParam);
   }
   return DefWindowProc (hWnd, nMsg, wParam, lParam);
}

//==============================================================================
// Funktion der Client-Area, wird von WndProc aufgerufen
//==============================================================================
#pragma argsused
LRESULT fnWMPaint (HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   HDC         hDC ;
   PAINTSTRUCT aPS ;
   RECT        aRectTitel, aRectZeit, aClientRect;
   char        sTime [10] = "";
   struct time aTime;
   char        sUeberschrift [2*EINHEITTEXTLEN + 50];
   HBRUSH      hBlueBrush;
   HFONT       hFont, hOldFont;
   int         nOldBk;
   COLORREF    nOldTxtCol;

   GetClientRect (hWnd, &aClientRect);

   //---------------------------------------------------------------------------
   // Zeitanzeige
   //---------------------------------------------------------------------------
   gettime(&aTime);
   sprintf (sTime, "%02d:%02d.%02d", aTime.ti_hour, aTime.ti_min, aTime.ti_sec);
   aRectZeit = aClientRect;
   aRectZeit.top = aRectZeit.bottom - 1.5 * nCharHeight;

   //---------------------------------------------------------------------------
   // berschrift
   //---------------------------------------------------------------------------
   hBlueBrush = CreateSolidBrush (RGB (100, 100, 255));
   hFont      = CreateFont (nCharHeight, 0, 0, 0, FW_THIN, FALSE, FALSE, FALSE,
                            ANSI_CHARSET, OUT_DEFAULT_PRECIS,
                            CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
                            DEFAULT_PITCH, "Braggadocio");
   sprintf (sUeberschrift, "Umrechnung von %s nach %s", sEinheit1, sEinheit2);
   aRectTitel = aClientRect;
   aRectTitel.bottom = aRectTitel.top + 1.5 * nCharHeight;

   //---------------------------------------------------------------------------
   hDC = BeginPaint (hWnd, &aPS);

   FillRect (hDC, &aRectTitel, hBlueBrush);
   FrameRect(hDC, &aRectTitel, (HBRUSH)GetStockObject (BLACK_BRUSH));

   hOldFont   = (HFONT)SelectObject (hDC, hFont);
   nOldBk     = SetBkMode (hDC, TRANSPARENT);
   nOldTxtCol = SetTextColor (hDC, RGB (255,255,0));
   DrawText (hDC, sUeberschrift, -1, &aRectTitel, DT_SINGLELINE|DT_CENTER|DT_VCENTER);
   SetTextColor (hDC, nOldTxtCol);
   SetBkMode    (hDC, nOldBk);
   SelectObject (hDC, hOldFont);

   fnValueTable(hDC);

   FillRect (hDC, &aRectZeit, (HBRUSH)GetStockObject (LTGRAY_BRUSH));
   FrameRect(hDC, &aRectZeit, (HBRUSH)GetStockObject (BLACK_BRUSH));
   DrawText (hDC, sTime, -1, &aRectZeit, DT_SINGLELINE|DT_RIGHT|DT_VCENTER);

   EndPaint (hWnd, &aPS);

   //---------------------------------------------------------------------------
   // Limitierte Resourcen wieder freigeben
   //---------------------------------------------------------------------------
   DeleteObject (hBlueBrush);
   DeleteObject (hFont);

   return 0;
}

//==============================================================================
// Funktion der Client-Area, wird von WndProc aufgerufen
//==============================================================================
#pragma argsused
LRESULT fnWMCreate (HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   TEXTMETRIC aTM;

   HDC hDC = GetDC (hWnd) ;
   GetTextMetrics (hDC, &aTM) ;
   ReleaseDC (hWnd, hDC) ;

   nCharHeight     = aTM.tmHeight + aTM.tmExternalLeading;
   nSmallCharWidth = aTM.tmAveCharWidth;
   nBigCharWidth   = (aTM.tmPitchAndFamily & TMPF_FIXED_PITCH ? 3 : 2) *
                      aTM.tmAveCharWidth / 2;
   return 0;
}

//==============================================================================
// Funktion der Client-Area, wird von WndProc aufgerufen
//==============================================================================
#pragma argsused
LRESULT fnWMDestroy (HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   PostQuitMessage (0);
   KillTimer (hWnd, ID_TIMER);
   return 0;
}

//==============================================================================
// Funktion der Client-Area, wird von WndProc aufgerufen
//==============================================================================
#pragma argsused
LRESULT fnWMSize (HWND hWnd, WPARAM wParam, LPARAM lParam)
{
  int nActWidth  = LOWORD(lParam);
  int nMinWidth  = 50 * nSmallCharWidth;
  int nActHeight = HIWORD(lParam);
  int nMinHeight =  7 * nCharHeight;
  int nNewWidth  = nActWidth  < nMinWidth  ? nMinWidth  : nActWidth;
  int nNewHeight = nActHeight < nMinHeight ? nMinHeight : nActHeight;

  if ((nActWidth < nNewWidth) || (nActHeight < nNewHeight))
  {
    nNewWidth  = nNewWidth  + GetSystemMetrics (SM_CXFRAME) * 2;
    nNewHeight = nNewHeight + GetSystemMetrics (SM_CYFRAME) * 2
                            + GetSystemMetrics (SM_CYMENU)
                            + GetSystemMetrics (SM_CYCAPTION);
    MoveWindow(hWnd, 0, 0, nNewWidth, nNewHeight, TRUE);
  }

  //----------------------------------------------------------------------------
  // globale variablen setzten
  //----------------------------------------------------------------------------
  nAnzZeichen = nNewWidth  / nSmallCharWidth - 2;
  nAnzZeilen  = nNewHeight / nCharHeight     - 4;

  return 0;
}

//==============================================================================
// Funktion der Client-Area, wird von WndProc aufgerufen
//==============================================================================
#pragma argsused
LRESULT fnWMTimer (HWND hWnd, WPARAM wParam, LPARAM lParam)
{
  RECT aRect;
  GetClientRect (hWnd, &aRect);
  aRect.left = aRect.right   - nSmallCharWidth * 10;
  aRect.top  = aRect.bottom - nCharHeight;
  InvalidateRect (hWnd, &aRect, TRUE);
  return 0;
}

//==============================================================================
// Funktion zur Umrechnung von Werten
//==============================================================================
double fnCalcValue (double fValue, double fFaktor, bool bMultiply,
                    double fOffset, bool bAddOffset, bool bOffsetFirst)
{
   if (fFaktor != 0.0)
   {
     if (bOffsetFirst)
     {
        fValue = fValue + (bAddOffset ? fOffset : -fOffset);
        fValue = fValue * (bMultiply  ? fFaktor : 1/fFaktor);
     }
     else
     {
        fValue = fValue * (bMultiply  ? fFaktor : 1/fFaktor);
        fValue = fValue + (bAddOffset ? fOffset : -fOffset);
     }
   }
   else
   {
      fValue = 0.0;
   }
   return fValue;
}

//==============================================================================
// Funktion zur Umrechnung von Werten
//==============================================================================
void fnValueTable (HDC hDC)
{
   int nLine = 0;
   int nPos;
   int j = 0;
   double f = 0.0;
   char sText [100]    = "";
   UINT nOldSettings = SetTextAlign (hDC, TA_RIGHT | TA_TOP);

   while (((j+1)*23) < nAnzZeichen)
   {
      nLine = 0;
      nPos = nSmallCharWidth + j * 23 * nSmallCharWidth;
      for (nLine=0; nLine<nAnzZeilen; f+=0.5, nLine++)
      {
         sprintf (sText, "%10.2lf =", f);
         TextOut (hDC, nPos+12*nSmallCharWidth, (nLine+2)*nCharHeight, sText, strlen (sText));

         sprintf (sText, "%10.2lf", fnCalcValue(f, 2.56));
         TextOut (hDC, nPos+23*nSmallCharWidth, (nLine+2)*nCharHeight, sText, strlen (sText));
      }
      j++;
   }
   SetTextAlign (hDC, nOldSettings);
}

